@@ -29,14 +29,13 @@
  * then
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include "galias.h"
 #include "glib.h"
 #include "gprintfint.h"
 
@@ -186,7 +185,10 @@ g_on_error_stack_trace (const gchar *prg_name)
     ;
   glib_on_error_halt = TRUE;
 #else
-  abort ();
+  if (IsDebuggerPresent ())
+    G_BREAKPOINT ();
+  else
+    abort ();
 #endif
 }
 
