@@ -28,14 +28,14 @@
  * MT safe
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
+#include "galias.h"
+#include "glibconfig.h"
 
-#include "glib.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
+
 #ifndef G_OS_WIN32
 #include <sys/time.h>
 #include <time.h>
@@ -46,9 +46,10 @@
 #include <windows.h>
 #endif /* G_OS_WIN32 */
 
-typedef struct _GRealTimer GRealTimer;
+#include "glib.h"
 
-struct _GRealTimer
+
+struct _GTimer
 {
 #ifdef G_OS_WIN32
   DWORD start;
@@ -72,14 +73,14 @@ struct _GRealTimer
 GTimer*
 g_timer_new (void)
 {
-  GRealTimer *timer;
+  GTimer *timer;
 
-  timer = g_new (GRealTimer, 1);
+  timer = g_new (GTimer, 1);
   timer->active = TRUE;
 
   GETTIME (timer->start);
 
-  return ((GTimer*) timer);
+  return timer;
 }
 
 void
@@ -93,46 +94,87 @@ g_timer_destroy (GTimer *timer)
 void
 g_timer_start (GTimer *timer)
 {
-  GRealTimer *rtimer;
-
   g_return_if_fail (timer != NULL);
 
-  rtimer = (GRealTimer*) timer;
-  rtimer->active = TRUE;
+  timer->active = TRUE;
 
-  GETTIME (rtimer->start);
+  GETTIME (timer->start);
 }
 
 void
 g_timer_stop (GTimer *timer)
 {
-  GRealTimer *rtimer;
-
   g_return_if_fail (timer != NULL);
 
-  rtimer = (GRealTimer*) timer;
-  rtimer->active = FALSE;
+  timer->active = FALSE;
 
-  GETTIME(rtimer->end);
+  GETTIME(timer->end);
 }
 
 void
 g_timer_reset (GTimer *timer)
 {
-  GRealTimer *rtimer;
-
   g_return_if_fail (timer != NULL);
 
-  rtimer = (GRealTimer*) timer;
+  GETTIME (timer->start);
+}
 
-  GETTIME (rtimer->start);
+void
+g_timer_continue (GTimer *timer)
+{
+#ifdef G_OS_WIN32
+  DWORD elapsed;
+#else
+  struct timeval elapsed;
+#endif /* G_OS_WIN32 */
+
+  g_return_if_fail (timer != NULL);
+  g_return_if_fail (timer->active == FALSE);
+
+  /* Get elapsed time and reset timer start time
+   *  to the current time minus the previously
+   *  elapsed interval.
+   */
+
+#ifdef G_OS_WIN32
+
+  elapsed = timer->end - timer->start;
+
+  GETTIME (timer->start);
+
+  timer->start -= elapsed;
+
+#else /* !G_OS_WIN32 */
+
+  if (timer->start.tv_usec > timer->end.tv_usec)
+    {
+      timer->end.tv_usec += G_USEC_PER_SEC;
+      timer->end.tv_sec--;
+    }
+
+  elapsed.tv_usec = timer->end.tv_usec - timer->start.tv_usec;
+  elapsed.tv_sec = timer->end.tv_sec - timer->start.tv_sec;
+
+  GETTIME (timer->start);
+
+  if (timer->start.tv_usec < elapsed.tv_usec)
+    {
+      timer->start.tv_usec += G_USEC_PER_SEC;
+      timer->start.tv_sec--;
+    }
+
+  timer->start.tv_usec -= elapsed.tv_usec;
+  timer->start.tv_sec -= elapsed.tv_sec;
+
+#endif /* !G_OS_WIN32 */
+
+  timer->active = TRUE;
 }
 
 gdouble
 g_timer_elapsed (GTimer *timer,
 		 gulong *microseconds)
 {
-  GRealTimer *rtimer;
   gdouble total;
 #ifndef G_OS_WIN32
   struct timeval elapsed;
@@ -140,39 +182,37 @@ g_timer_elapsed (GTimer *timer,
 
   g_return_val_if_fail (timer != NULL, 0);
 
-  rtimer = (GRealTimer*) timer;
-
 #ifdef G_OS_WIN32
-  if (rtimer->active)
-    rtimer->end = GetTickCount ();
+  if (timer->active)
+    timer->end = GetTickCount ();
 
   /* Check for wraparound, which happens every 49.7 days. */
-  if (rtimer->end < rtimer->start)
-    total = (UINT_MAX - (rtimer->start - rtimer->end)) / 1000.0;
+  if (timer->end < timer->start)
+    total = (UINT_MAX - (timer->start - timer->end - 1)) / 1000.0;
   else
-    total = (rtimer->end - rtimer->start) / 1000.0;
+    total = (timer->end - timer->start) / 1000.0;
 
   if (microseconds)
     {
-      if (rtimer->end < rtimer->start)
+      if (timer->end < timer->start)
 	*microseconds =
-	  ((UINT_MAX - (rtimer->start - rtimer->end)) % 1000) * 1000;
+	  ((UINT_MAX - (timer->start - timer->end - 1)) % 1000) * 1000;
       else
 	*microseconds =
-	  ((rtimer->end - rtimer->start) % 1000) * 1000;
+	  ((timer->end - timer->start) % 1000) * 1000;
     }
 #else /* !G_OS_WIN32 */
-  if (rtimer->active)
-    gettimeofday (&rtimer->end, NULL);
+  if (timer->active)
+    gettimeofday (&timer->end, NULL);
 
-  if (rtimer->start.tv_usec > rtimer->end.tv_usec)
+  if (timer->start.tv_usec > timer->end.tv_usec)
     {
-      rtimer->end.tv_usec += G_USEC_PER_SEC;
-      rtimer->end.tv_sec--;
+      timer->end.tv_usec += G_USEC_PER_SEC;
+      timer->end.tv_sec--;
     }
 
-  elapsed.tv_usec = rtimer->end.tv_usec - rtimer->start.tv_usec;
-  elapsed.tv_sec = rtimer->end.tv_sec - rtimer->start.tv_sec;
+  elapsed.tv_usec = timer->end.tv_usec - timer->start.tv_usec;
+  elapsed.tv_sec = timer->end.tv_sec - timer->start.tv_sec;
 
   total = elapsed.tv_sec + ((gdouble) elapsed.tv_usec / 1e6);
   if (total < 0)
@@ -200,7 +240,7 @@ g_usleep (gulong microseconds)
   struct timespec request, remaining;
   request.tv_sec = microseconds / G_USEC_PER_SEC;
   request.tv_nsec = 1000 * (microseconds % G_USEC_PER_SEC);
-  while (nanosleep (&request, &remaining) == EINTR)
+  while (nanosleep (&request, &remaining) == -1 && errno == EINTR)
     request = remaining;
 # else /* !HAVE_NANOSLEEP */
   if (g_thread_supported ())
@@ -241,36 +281,36 @@ g_usleep (gulong microseconds)
 
 /**
  * g_time_val_add:
- * @time: a #GTimeVal
+ * @time_: a #GTimeVal
  * @microseconds: number of microseconds to add to @time
  *
- * Adds the given number of microseconds to @time. @microseconds can
- * also be negative to decrease the value of @time.
+ * Adds the given number of microseconds to @time_. @microseconds can
+ * also be negative to decrease the value of @time_.
  **/
 void 
-g_time_val_add (GTimeVal *time, glong microseconds)
+g_time_val_add (GTimeVal *time_, glong microseconds)
 {
-  g_return_if_fail (time->tv_usec >= 0 && time->tv_usec < G_USEC_PER_SEC);
+  g_return_if_fail (time_->tv_usec >= 0 && time_->tv_usec < G_USEC_PER_SEC);
 
   if (microseconds >= 0)
     {
-      time->tv_usec += microseconds % G_USEC_PER_SEC;
-      time->tv_sec += microseconds / G_USEC_PER_SEC;
-      if (time->tv_usec >= G_USEC_PER_SEC)
+      time_->tv_usec += microseconds % G_USEC_PER_SEC;
+      time_->tv_sec += microseconds / G_USEC_PER_SEC;
+      if (time_->tv_usec >= G_USEC_PER_SEC)
        {
-         time->tv_usec -= G_USEC_PER_SEC;
-         time->tv_sec++;
+         time_->tv_usec -= G_USEC_PER_SEC;
+         time_->tv_sec++;
        }
     }
   else
     {
       microseconds *= -1;
-      time->tv_usec -= microseconds % G_USEC_PER_SEC;
-      time->tv_sec -= microseconds / G_USEC_PER_SEC;
-      if (time->tv_usec < 0)
+      time_->tv_usec -= microseconds % G_USEC_PER_SEC;
+      time_->tv_sec -= microseconds / G_USEC_PER_SEC;
+      if (time_->tv_usec < 0)
        {
-         time->tv_usec += G_USEC_PER_SEC;
-         time->tv_sec--;
+         time_->tv_usec += G_USEC_PER_SEC;
+         time_->tv_sec--;
        }      
     }
 }
